package ir.apdroid.wi_firemote;

import android.app.Activity;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.net.DhcpInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.format.Formatter;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;

import java.io.IOException;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.nio.ByteOrder;

public class MainActivity extends Activity {
    TextView tv;
    Button onoff;
    boolean connected = false;
    MediaPlayer mp_on, mp_off;
    AudioManager am;
    LinearLayout switch_container;
    static ImageView lamp;
    String Module_IP;

    static final String NICKNAME = "Wei-Meng";
    //---socket---
    InetAddress serverAddress;
    Socket socket;


    //---thread for communicating on the socket---
    CommsThread commsThread;

    //---used for updating the UI on the main activity---
    static Handler UIupdater = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            int numOfBytesReceived = msg.arg1;
            byte[] buffer = (byte[]) msg.obj;

            //---convert the entire byte array to string---
            String strReceived = new String(buffer);

            //---extract only the actual string received---
            strReceived = strReceived.substring(
                    0, numOfBytesReceived);


            if ( strReceived.contains("RELAY TURN ON") ){
                lamp.setImageResource(R.drawable.l_on);
            }

            if ( strReceived.contains("RELAY TURN OFF") ){
                lamp.setImageResource(R.drawable.l_off);
            }
            //---display the text received on the TextView---
           //txtMessagesReceived.setText(
            //        txtMessagesReceived.getText().toString() +
            //                strReceived);
        }
    };



    private class CreateCommThreadTask extends AsyncTask<Void, Integer, Void> {
        @Override
        protected Void doInBackground(Void... params) {
            try {
                //---create a socket---
                serverAddress =
                        InetAddress.getByName(Module_IP);
                socket = new Socket(serverAddress, 1394);
                commsThread = new CommsThread(socket);
                commsThread.start();
                //---sign in for the user; sends the nick name---
                sendToServer(NICKNAME);
                connected = true;
            } catch (UnknownHostException e) {
                Log.d("Sockets", e.getLocalizedMessage());
            } catch (IOException e) {
                Log.d("Sockets", e.getLocalizedMessage());
            }
            return null;
        }
    }



    private class WriteToServerTask extends AsyncTask
            <byte[], Void, Void> {
        protected Void doInBackground(byte[]...data) {
            commsThread.write(data[0]);
            return null;
        }
    }

    private class CloseSocketTask extends AsyncTask
            <Void, Void, Void> {
        //@Override
        protected Void doInBackground(Void... params) {
            try {
                if(connected){
                    socket.close();
                }

            } catch (Exception e) {
                //Log.d("Sockets", e.getLocalizedMessage());
                Toast.makeText(getBaseContext(),"ErrInside",Toast.LENGTH_SHORT).show();
            }
            return null;
        }
    }



    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_NOSENSOR);
        setContentView(R.layout.activity_main);

        init();
        main();
    }

    public void init(){
        tv = (TextView)findViewById(R.id.textView);
        //onoff=(Button)findViewById(R.id.button2);


        am = (AudioManager) getSystemService(AUDIO_SERVICE);
        mp_on = MediaPlayer.create(this, R.raw.on);
        mp_off = MediaPlayer.create(this, R.raw.off);

        lamp = (ImageView) findViewById(R.id.imageView);

        switch_container =(LinearLayout) findViewById(R.id.key);
        //---get the views---

    }

    public void main(){
        //tv.setText(wifiIpAddress(this) );

        final WifiManager manager = (WifiManager) super.getSystemService(WIFI_SERVICE);
        final DhcpInfo dhcp = manager.getDhcpInfo();
        final String address = Formatter.formatIpAddress(dhcp.gateway);
        tv.setText(address+":1394");
        Module_IP = address;

        try{
            new CreateCommThreadTask().execute();
        }catch(Exception e){
            Log.d("Sockets","onResomeError");
        }

    }


    public void on_click(View view) {
        //Toast.makeText(this,"on",Toast.LENGTH_SHORT).show();
        //play_on();
        switch_container.setBackgroundResource(R.drawable.s_on);

        if(connected) {
            sendToServer("on");
        }else{
            Toast.makeText(this,"اتصال برقرار نیست",Toast.LENGTH_SHORT).show();
        }


    }

    public void off_click(View view) {
        //Toast.makeText(this,"off",Toast.LENGTH_SHORT).show();
        //play_off();
        switch_container.setBackgroundResource(R.drawable.s_off);

        if(connected) {
            sendToServer("off");
        }else{
            Toast.makeText(this,"اتصال برقرار نیست",Toast.LENGTH_SHORT).show();
        }

    }

    public void onof(View view) {
        onoff.setText("Turning on...");

        try{
            new CloseSocketTask().execute();
        }catch(Exception e){
           // Log.d("Sockets","onPauseError");
            Toast.makeText(this,"Err",Toast.LENGTH_SHORT).show();
        }

    }



    private void sendToServer(String message) {
        byte[] theByteArray =
                message.getBytes();
        new WriteToServerTask().execute(theByteArray);
    }

    @Override
    public void onResume() {
        super.onResume();
        /*
        try{
            new CreateCommThreadTask().execute();
        }catch(Exception e){
            Log.d("Sockets","onResomeError");
        }
*/
    }

    @Override
    public void onPause() {
        super.onPause();

        try{
            new CloseSocketTask().execute();
            //if(connected){
                //socket.close();
            //}

        }catch(Exception e){
            Log.d("Sockets","onPauseError:"+e);
        }

    }

    protected String wifiIpAddress(Context context) {
        WifiManager wifiManager = (WifiManager) context.getSystemService(WIFI_SERVICE);
        int ipAddress = wifiManager.getConnectionInfo().getIpAddress();

        // Convert little-endian to big-endianif needed
        if (ByteOrder.nativeOrder().equals(ByteOrder.LITTLE_ENDIAN)) {
            ipAddress = Integer.reverseBytes(ipAddress);
        }

        byte[] ipByteArray = BigInteger.valueOf(ipAddress).toByteArray();

        String ipAddressString;
        try {
            ipAddressString = InetAddress.getByAddress(ipByteArray).getHostAddress();
        } catch (UnknownHostException ex) {
            Log.e("WIFIIP", "Unable to get host address.");
            ipAddressString = "Not Connect!";
        }

        return ipAddressString;
    }



    //*******************************************************
    public void play_on(){
        try {
            release_all();
            mp_on.start();
        } catch(Exception e) { e.printStackTrace(); }
    }

    public void play_off(){
        try {
            release_all();
            mp_off.start();
        } catch(Exception e) { e.printStackTrace(); }
    }

    public void release_all(){
        if (mp_on.isPlaying()) {
            mp_on.stop();
            mp_on.release();
            mp_on = MediaPlayer.create(this, R.raw.on);
        }
        if (mp_off.isPlaying()) {
            mp_off.stop();
            mp_off.release();
            mp_off = MediaPlayer.create(this, R.raw.off);
        }
    }


    //*******************************************
    // clear extra object when go to next activity or send activity to ram
    public void onStop(){
        super.onStop();
        mp_on.release();
        mp_off.release();
    }

}
